//Breakable Windows For Multiplayer by n00bCODmapper
//Adapted for Cod2 by tltrude and Bodger2

//Mapping

// The triggers must have an origin brush in the center.  Select both brushes and then turn them into a trigger_damage.
// Retexture the origin brush.
// The triggers are all named "window" and are slightly thicker then the windows they are for.
// Create two script_brushmodels.  One for the unbroken window, and one for the broken window.
//     Note: The broken window is optional.  The unbroken window must also have a center origin brushes as part of the entity.
// Texture all window edges with nodraw -- to allow bullets to pass through.  If you use caulk edges, bullets will be blocked until the window breaks.
// Each trigger must target (key w) an unbroken window.
// Each unbroken window must target (key w) a broken window -- if one is used.
// If you wish, add a playerclip textured brush to block players after the window breaks.
// Place all entities in the same spot. 
// The entities can be cloned, but make sure all the parts are selected and all (except the trigger) have "auto#" targetnames.

//Scripting
 
// In your main script add this line, " maps\mp\_breakwindow::window_think(); " -- just above the "_load::main()" line.
// If you wish, put these damage adjustment lines below the setcvar lines (this step is optional, defaults at 95/150). 
//	level.windamage_min = 95;  // windows break if a single hit exceeds this amount.
//	level.windamage_max = 150; // windows break when this much damage is accumulated.
// Also, place this line at the very top, " #include maps\mp\_utility; ".
// For sound you must add the following lines to your map's main/soundaliases/yourmapname.csv file.
//  	window_break,1,glass/glass05.wav,0.8,1,na,0.9,1.05,120,800,,streamed,,,,all_mp
//  	window_break,2,glass/glass08.wav,0.8,1,na,0.9,1.05,120,800,,streamed,,,,all_mp

window_think()
{
	windows = getentarray("window","targetname"); //find everything with targetname "window"
	if(isdefined(windows)) //if there were window triggers found
	{
		windowfx = loadfx ("fx/glass/glass_break4.efx"); //define the glass effect
		for(i=0;i<windows.size;i++) //start threads listening for all window triggers
		{
			windows[i] thread window(windowfx);
		}
	}
	//end breaking windows main
}

window(windowfx) //This determines where, when, and how to play the glass breaking effect
{
	if(!isdefined(self.target)) //see if window trigger still exists
	{
		iprintln("no target key found for window trigger at ", self getorigin());
		return;
	}

	window = getent(self.target,"targetname"); //get the targetname for the unbroken window

	if(!isdefined(window)) //see if trigger has target (the unbroken window)
	{
		iprintln("window not found for window trigger at ", self getorigin());
		return;
	}

	if(!isdefined(window.origin)) //check for origin key on window
	{
		iprintln("origin key not found for window at ", self getorigin());
		return;	
	}

	self enableGrenadeTouchDamage();  //grenade impact will break window
	window disableGrenadeBounce();      //so you can throw grenades through
		
	brokenwindow = undefined;
	if(isdefined(window.target)) //see if unbroken window has a target.
	{
		brokenwindow = getent(window.target,"targetname"); //get the targetname for brokenwindow.
	}

	org = spawn("script_origin",window.origin); //copy of origin of this trigger
	sampledist = 1; //accuracy in determiniting size of window
	spacing = 24; //distance between each breaking glass effect

	while(org istouching(window)) //determines end of trigger along x axis to nearest 1 unit, windows that are not aligned with an axis will not work because of this.
	{
		org.origin += (sampledist,0,0);
	}
	xendorg = org.origin; //record largest whole x value
	xdist = org.origin[0]-window.origin[0]; //record size along x axis
	org.origin = window.origin;

	while(org istouching(window)) //determines end of trigger along y axis to nearest 1 unit
	{
		org.origin += (0,sampledist,0);
	}
	yendorg = org.origin; //record largest whole y value
	ydist = org.origin[1]-window.origin[1]; //record size along y axis
	org.origin = window.origin;

	while(org istouching(window)) //determines end of trigger along z axis to nearest 1 unit
	{
		org.origin += (0,0,sampledist);
	}
	zendorg = org.origin; //record largest whole z value
	zdist = org.origin[2]-window.origin[2]; //record size along z axis

	point = [];  //will hold all of the points to start glass effects

	if(ydist>xdist) //window is aligned along the x axis, windows running along z axis will not work because of this.
	{		
		direction = vectornormalize(xendorg-window.origin); //creates a vector perpendicular to the window for the width of the effect. 
		//Note that the width of the trigger will affect how far glass flies.
		ycount = (int(ydist)/spacing); //determines how many effects along y axis (rounds down)
		if(!(ycount))
			ycount = 1; //must have at least 1
		zcount = (int(zdist)/spacing); //determines how many effects along z axis (rounds down)
		if(!(zcount))
			zcount = 1; //must have at least 1
		ystart = window.origin[1]+((ydist - (ycount*spacing))/2); //Since the effect has a pretetermined size, make the amount of error equal on both sides.
		zstart = window.origin[2]+((zdist - (zcount*spacing))/2); //Since the effect has a pretetermined size, make the amount of error equal on both sides.
		//Determine where center points are for glass effect.
		for(x=0;x<ycount;x++)
		{
			for(y=0;y<zcount;y++)
			{
				point[x][y] = (window.origin[0], ystart+(spacing*x), zstart+(spacing*y));
			}
		}

	}
	else  //window is aligned along the x axis
	{
		direction = vectornormalize(yendorg-window.origin); //creates a vector perpendicular to the window for the width of the effect.
		xcount = (int(xdist)/spacing); //determines how many effects along x axis (rounds down)
		if(!(xcount))
			xcount = 1; //must have at least 1
		zcount = (int(zdist)/spacing); //determines how many effects along z axis (rounds down)
		if(!(zcount))
			zcount = 1; //must have at least 1
		xstart = window.origin[0]+((xdist - (xcount*spacing))/2); //Since the effect has a pretetermined size, make the amount of error equal on both sides.
		zstart = window.origin[2]+((zdist - (zcount*spacing))/2); //Since the effect has a pretetermined size, make the amount of error equal on both sides.
		//Determine where center points are for glass effect.
		for(x=0;x<xcount;x++)
		{
			for(y=0;y<zcount;y++)
			{
				point[x][y] = (xstart+(spacing*x), window.origin[1], zstart+(spacing*y));
			}

		}	
	}
	org delete(); //free memory
	window show(); //show unbroken window

	//windowclip = undefined;
	//if((isdefined(brokenwindow)) && (isdefined(brokenwindow.target))) //see if brokenwindow is there and has a target.
	//{
	//	windowclip = getent(brokenwindow.target,"targetname"); //get the targetname for window clip brush.
	//	window notsolid();  //Allows bullets to pass through unbroken window.
	//}

	if(isdefined(brokenwindow))
	{
		brokenwindow hide(); //make broken window disappear.
		brokenwindow notsolid(); //allow player to move through broken window.
	}

	breaktype = self breakwait(); //Function is defined below. There it waits for window break event.

	if(isdefined(brokenwindow))
	{
		brokenwindow show(); //show broken window
	}
	window hide(); //hide unbroken window
	if(breaktype != "smallbreak")  //bigbreak
	{
		direction = vectormultiply(direction,5); //enhance the distance particles fly
	}
		
	//Use the info gathered earlier to play the effects
	for(i=0;i<point.size;i++)
	{	
		for(j=0;j<point[i].size;j++)
		{
			//iprintln("point = " + point[i][j]); // <----Add for debug.
			//iprintln("window origin = " + window.origin); // <----Add for debug.

			playfx(windowfx,point[i][j],direction);	//this is the money shot
			window playsound ("window_break");			
		}	
	}	
	wait 2; //allow effects to finish playing out before thread ends
	window delete(); //remove unbroken window
	self delete(); //remove trigger

	//if(isdefined(windowclip))
	//{
	//	windowclip delete();
	//}
}

// Determine big or small break, affecting how far glass flies
breakwait()
{
   	totaldamage = 0;
    	while(1) // Thread hangs here waiting for damage to the trigger
    	{
		self waittill ("damage",amount);
		self playsound ("window_break");
		if(!isdefined(level.windamage_min))
			level.windamage_min = 95;
		if(!isdefined(level.windamage_max))
			level.windamage_max = 150;
		//iprintlnbold("Window damage = ", amount); // <--- Add this line for debug.
		if(amount > level.windamage_min)
		{
			return ("bigbreak");
		}
		totaldamage += amount; // window accumulates damage
		//iprintlnbold("Total damage = ", totaldamage); // <--- Add this line for debug.
		if(totaldamage > level.windamage_max) // The trigger has been set off
		break;
    	}
    	return ("smallbreak");
}

vectormultiply (vec, dif) //used to enhance the direction vector so glass flies farther
{
	vec = (vec[0] * dif, vec[1] * dif, vec[2] * dif);
	return vec;
}

// end